'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.parse = parse;

var _internals = require('./internals');

var _core = require('./core');

var _combinators = require('./combinators');

/**
 * Executes a parser and returns the result.
 *
 * @param {Function} parser - The parser to execute.
 * @param {String} string - String to parse.
 * @param {Object} [opts] - Options object.
 * @param {Boolean} [opts.throwOnError=true] - If truthy, throws a ParserError
 *                                             if the parser fails and returns
 *                                             ParserState instead of its value.
 * @param {String} [opts.fileName] - filename to use for error messages.
 * @memberof module:mona/api
 * @instance
 *
 * @example
 * parse(token(), 'a') // => 'a'
 */
function parse(parser, string) {
  var opts = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

  opts.throwOnError = typeof opts.throwOnError === 'undefined' ? true : opts.throwOnError;
  if (!opts.allowTrailing) {
    parser = (0, _combinators.followedBy)(parser, (0, _core.eof)());
  }
  var parserState = (0, _internals.invokeParser)(parser, new _internals.ParserState(undefined, string, 0, opts.userState, opts.position || new _internals.SourcePosition(opts.fileName), false));
  if (parserState.failed && opts.throwOnError) {
    throw parserState.error;
  } else if (parserState.failed && !opts.throwOnError) {
    return parserState.error;
  } else if (!parserState.failed && !opts.throwOnError) {
    return parserState;
  } else if (opts.returnState) {
    return parserState;
  } else {
    return parserState.value;
  }
}